#! ruby
require 'phi'
class OutlineForm < Phi::Form
  def add_entries(node, dir)
    Dir.entries(dir).each do |ent|
      next if ent =~ /^\./
      next if ent =~ /\~$/
      n = tv.items.add_child node, ent
      d = File.join(dir, ent)
      n.data = d
      if test ?d, d
        add_entries n, d
      elsif test ?f, d
        if ent =~ /\.rb$/
          n.image_index = 2
          n.selected_index = 2
        else
          n.image_index = 3
          n.selected_index = 3
        end
      end
    end
  end
  def refresh_tree()
    tv.items.clear
    tv.items.update do
      root = tv.items.add nil, 'wdp'
      dir = 'c:/var/www/wdp'
      root.data = dir
      add_entries(root, dir)
      tv.full_expand
    end
    tv.selected = tv.items.first_node
    nil
  end
  def initialize
    self.left = 0
    self.top = 0
    default_width = 216
    self.width = default_width
    self.height = 800-100-4
    p rect
    self.caption = "Outline"
    self.font.name = "Terminal"
    self.font.size = 14
    self.font.style = [Phi::FS_BOLD]

    require 'win32ole'
    @vim = WIN32OLE.new('Vim.Application')
    @vim.SendKeys("<C-\\><C-N>:source c:/tmp/kof.vim<Enter>")
    @vim.SendKeys("<C-\\><C-N>:winp #{right} #{top}<Enter>")

    il = Phi::ImageList.new
    il.add_icon Phi::Icon.new.load('ico/folder_close.ico')
    il.add_icon Phi::Icon.new.load('ico/folder_open.ico')
    il.add_icon Phi::Icon.new.load('ico/ruby.ico')
    il.add_icon Phi::Icon.new.load('ico/note.ico')
    #Phi::ScrollBox.new(self, :sb) do
    #  self.align = Phi::AL_CLIENT
    #end
    Phi::TreeView.new(self, :tv) do
      self.align = Phi::AL_CLIENT
    end
    self.control_attr_flatten
    tv.images = il
    tv.hide_selection = false
    tv.show_buttons = false
    tv.show_lines = false
    refresh_tree()
    tv.on_key_down = proc do |sender, key, shift|
      #p [:on_key_down, sender, key, shift]
      if key == 13
        vim_edit tv.selected.data
        return 0
      elsif key == 39 && shift.include?(0)
        self.width*=2
        #puts "width=#{width}"
        return 0
      elsif key == 37 && shift.include?(0)
        w = self.width
        w/=2
        w = default_width if w < default_width
        self.width = w
        #puts "width=#{width}"
        return 0
      elsif key == 38 && shift.include?(0)
        self.height-=8
        return 0
      elsif key == 40 && shift.include?(0)
        self.height+=8
        return 0
      elsif key == 116  # [F5] refresh tree
        refresh_tree
      elsif key == 120  # [F9] open webbrowser
        require 'browser'
        form = BrowserForm.new
        form.rect = Phi::Rect.new(right, top, 1024-128+8, 800-100-4)
        p form.rect
        form.location = "http://localhost:3000/"
        form.navigate
        form.show
      end
      key
    end
    tv.on_click = proc do |sender|
      vim_edit tv.selected.data
    end
    #tv.on_change = proc do |sender, node|
    #end
    self.on_resize = proc do |sender|
      @vim.SendKeys("<C-\\><C-N>:winp #{right} #{top}<Enter>")
    end
  end
  def vim_edit(path)
    @vim.SendKeys("<C-\\><C-N>:e #{path}<Enter>")
    # @vim.SetForeground
  end
end
if __FILE__ == $0
  form = OutlineForm.new do
  end
  form.show
  Phi.mainloop
end
